package at.ac.tuwien.sepm.assignment.individual.integration;

import static org.junit.jupiter.api.Assertions.*;

import at.ac.tuwien.sepm.assignment.individual.endpoint.dto.HorseDto;
import at.ac.tuwien.sepm.assignment.individual.enums.ERace;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.core.io.ClassPathResource;
import org.springframework.http.*;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.event.annotation.AfterTestMethod;
import org.springframework.test.context.jdbc.Sql;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import java.io.File;
import java.sql.Date;

@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
@ActiveProfiles("test")
public class HorseEndpointTest {
    @Value("${spring.upload.path}")
    private String FILE_BASE_PATH;
    private static final RestTemplate REST_TEMPLATE = new RestTemplate();
    private static final String BASE_URL = "http://localhost:";
    private static final String HORSE_URL = "/horses";

    @LocalServerPort
    private int port;

    @Test
    @DisplayName("Adding a new horse with the correct parameters will return HTTP 201 and the new HorseDto")
    public void addingNewHorse_correctParameters_shouldReturnStatus201AndHorse() {
        HorseDto newHorse = new HorseDto("Zephyr", "Nice horse", (short) 4, Date.valueOf("2020-01-01"), ERace.APPALOOSA, "files/test.png", null);

        HttpEntity<HorseDto> request = new HttpEntity<>(newHorse);
        ResponseEntity<HorseDto> response = REST_TEMPLATE
            .exchange(BASE_URL + port + HORSE_URL, HttpMethod.POST, request, HorseDto.class);

        // Compare everything except ids and timestamps
        assertEquals(response.getStatusCode(), HttpStatus.CREATED);
        assertEquals(newHorse.getName(), response.getBody().getName());
        assertEquals(newHorse.getDescription(), response.getBody().getDescription());
        assertEquals(newHorse.getScore(), response.getBody().getScore());
        assertEquals(newHorse.getBirthday().toString(), response.getBody().getBirthday().toString());
        assertEquals(newHorse.getImagePath(), response.getBody().getImagePath());
        assertEquals(newHorse.getOwner(), response.getBody().getOwner());
    }

    @Test
    @DisplayName("Updating a horse with the correct parameters will return 200 and the new HorseDto")
    public void updatingNewHorse_correctParameters_shouldReturnStatus200AndHorse() {
        HorseDto newHorse = new HorseDto("Zephyr", "Nice horse", (short) 4, Date.valueOf("2020-01-01"), ERace.APPALOOSA, "files/test.png", null);

        // Create a new horse
        HttpEntity<HorseDto> request = new HttpEntity<>(newHorse);
        ResponseEntity<HorseDto> response = REST_TEMPLATE
            .exchange(BASE_URL + port + HORSE_URL, HttpMethod.POST, request, HorseDto.class);

        // Update the horse
        newHorse.setId(response.getBody().getId());
        newHorse.setName("Katrina");
        newHorse.setDescription("Fast horse");
        newHorse.setScore((short) 3);
        newHorse.setBirthday(Date.valueOf("2005-01-01"));
        newHorse.setRace(ERace.MORGAN);
        newHorse.setImagePath("files/katrina.png");
        newHorse.setOwner(null);

        request = new HttpEntity<>(newHorse);
        response = REST_TEMPLATE
            .exchange(BASE_URL + port + HORSE_URL + '/' + newHorse.getId(), HttpMethod.PUT, request, HorseDto.class);

        // Compare everything except timestamps
        assertEquals(response.getStatusCode(), HttpStatus.OK);
        assertEquals(newHorse.getId(), response.getBody().getId());
        assertEquals(newHorse.getName(), response.getBody().getName());
        assertEquals(newHorse.getDescription(), response.getBody().getDescription());
        assertEquals(newHorse.getScore(), response.getBody().getScore());
        assertEquals(newHorse.getBirthday().toString(), response.getBody().getBirthday().toString());
        assertEquals(newHorse.getImagePath(), response.getBody().getImagePath());
        assertEquals(newHorse.getOwner(), response.getBody().getOwner());
    }

    @Test
    @DisplayName("Uploading an image in the correct format will return HTTP 201")
    public void addingNewImage_correctFormat_shouldReturnStatus201() {
        try {
            // More information:
            // https://www.baeldung.com/spring-rest-template-multipart-upload
            // https://github.com/spring-guides/gs-uploading-files/blob/master/complete/src/test/java/com/example/uploadingfiles/FileUploadIntegrationTests.java

            // Get the file
            ClassPathResource image = new ClassPathResource("horse.jpg", getClass());

            // Set the body
            MultiValueMap<String, Object> body = new LinkedMultiValueMap<>();
            body.add("file", image);

            // Create the HTTP Entity
            HttpEntity<MultiValueMap<String, Object>> request = new HttpEntity<>(body);

            ResponseEntity<String> response = REST_TEMPLATE.postForEntity(BASE_URL + port + HORSE_URL + "/upload", request, String.class);
            assertEquals(HttpStatus.CREATED, response.getStatusCode());
        } finally {
            new File(FILE_BASE_PATH + "horse.jpg").delete();
        }
    }
}
